#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "text.h"

// 
extern Display* dis;  // Default Display
extern int scr;  // Default Screen
extern Window rootwin;


static char *Hfont_default[] = {
	"rk14",
	"7x14",
	NULL};
static char *Zfont_default[] = {
	"k14",			/* 14x14 Kanji */
	"kanji16",		/* 16x16 Kanji */
	NULL};

static XFontStruct* fsH;
static XFontStruct* fsZ;
static int draw_mode = 0;


static int iskanji(int c)
{
	 if(0xa0 < c && c < 0xff) return 1;
	 return 0;
}


/*
========================================================================
ǽ: եȤθꥹȤˤɤäơեȤɤ
: list = ɤꥹ
: դäXFontStruct*, ̵NULL
­: 
========================================================================
*/
static XFontStruct *load_font(char**list)
{
	XFontStruct *p;
	char* name;
	
	while((name = (*list)++)!=NULL) {
		p = XLoadQueryFont(dis, name);
		if (p != NULL) return p;
	}
	return NULL;
}


/*
========================================================================
ǽ: ܸΤνԤ
: ʤ
: -1: 0:
­: dis / rootwin
      ǽ˰¹ԤƤ
========================================================================
*/
int setup_font()
{
	fsH  = load_font(Hfont_default);
	if (fsH==NULL) return -1;
	fsZ = load_font(Zfont_default);
	if (fsZ==NULL) {
		XFreeFont(dis, fsH);
		return -1;
	}
	return 0;
}


/*
========================================================================
ǽ: setup_fontǳդΤ
: ʤ
: ʤ
­: 
========================================================================
*/
void setdown_font()
{
	XFreeFont(dis, fsH);
	XFreeFont(dis, fsZ);
}


/*
========================================================================
ǽ: ֥롼 Ⱦʸ
: d = Drawable
      (x,y) = 
	  str = ʸ
	  len = Ĺ
: 褷֤
­: draw_mode˱ڤʬ
========================================================================
*/
static int draw_strH(GC gc, Drawable d,
					 int x, int y, unsigned char* str, int len)
{
	XSetFont(dis, gc, fsH->fid);
	if (draw_mode==0) {
		XDrawImageString(dis, d, gc,
						 x, y+fsH->max_bounds.ascent, (char *)str, len);
	} else {
		XDrawString(dis, d, gc,
					x, y+fsH->max_bounds.ascent, (char *)str, len);
	}
	len *= HWIDTH;
	return len;
}


/*
========================================================================
ǽ: ֥롼 ʸ
: d = Drawable
      (x,y) = 
	  str = ʸ
	  len = Ĺ
: 褷֤
­: draw_mode˱ڤʬ
========================================================================
*/
static int draw_strZ(GC gc, Drawable d,
					 int x, int y, unsigned char* str, int len)
{
	XChar2b buf[DRAWSTR_MAX];
	int i;

	XSetFont(dis, gc, fsZ->fid);
	len /= 2;
	for(i=0; i<len; i++) {
		buf[i].byte1 = *str++;
		buf[i].byte2 = *str++;
	}
	if (draw_mode==0) {
		XDrawImageString16(dis, d, gc,
						   x, y+fsZ->max_bounds.ascent, buf, len);
	} else {
		XDrawString16(dis, d, gc,
					  x, y+fsZ->max_bounds.ascent, buf, len);
	}
	len *= ZWIDTH;
	return len;
}


/*
========================================================================
ǽ: ܸʸ(EUC)
: d = Drawable
      (x,y) = ΰ
	  str = ʸ
: 褷(pixel)
­: 
========================================================================
*/
int draw_string(GC gc, Drawable d, int x, int y, char* str)
{
	unsigned char c, c2;
	int inkanji;
	int ct;
	int i, w;
	unsigned char *p;
	unsigned char buf[DRAWSTR_MAX];
	unsigned char buf2[DRAWSTR_MAX];

	strcpy((char*)buf, str);
	ct = w = 0;
	inkanji = 0;
	p = buf;
	while ((c = *p++) != '\0') {
		c2 = *p;
		if (iskanji(c)) {
			if (!inkanji) {
				i = draw_strH(gc, d, x, y, buf2, ct);
				w += i;
				x += i;
				ct = 0;
			}
			buf2[ct++] = c - 0x80;
			buf2[ct++] = c2 - 0x80;
			p++;
			inkanji = 1;
		}
		else if (c==SS2) {
			if (inkanji) {
				i = draw_strZ(gc, d, x, y, buf2, ct);
				w += i;
				x += i;
				ct = 0;
			}
			buf2[ct++] = c2;
			p++;
			inkanji = 0;
		}
		else {
			if (inkanji) {
				i = draw_strZ(gc, d, x, y, buf2, ct);
				w += i;
				x += i;
				ct = 0;
			}
			buf2[ct++] = c;
			inkanji = 0;
		}
	}
	if (ct) {
		if (inkanji==0) {
			i = draw_strH(gc, d, x, y, buf2, ct);
		} else {
			i = draw_strZ(gc, d, x, y, buf2, ct);
		}
		w += i;
	}
	return w;			/* string width (pixel) */
}


/*
========================================================================
ǽ: ƥȤ⡼ɤѹ
: m = 0:طʤ褹 1:ȥ󥹥ڥ
: ʤ
­: 
========================================================================
*/
void
set_text_mode(int m)
{
	draw_mode = m;
}


/*
========================================================================
ǽ: ƥȤοѹ
: pixel = 
: ʤ
­: 
========================================================================
*/
void
set_text_color(GC gc, unsigned long pixel)
{
	XSetForeground(dis, gc, pixel);
}


/*
========================================================================
ǽ: ƥȤطʿѹ
: pixel = 
: ʤ
­: 
========================================================================
*/
void
set_text_back_color(GC gc, unsigned long pixel)
{
	XSetBackground(dis, gc, pixel);
}


